function logpSet = ApplyLocalColorModel(labimSet, colorModelSet)
% estimate global color model
% labim - image patch in Lab space
% K - number of component in mixture model
% select - what pixels in patch belongs to region

vectIm = [];
for i = 1:numel(labimSet)
    labim = labimSet{i};
    imsiz = size(labim);

    imR = labim(:,:,1);
    imG = labim(:,:,2);
    imB = labim(:,:,3);

    fullSize = prod(imsiz(1:2));

    tempvectIm = [reshape(imR, [1 fullSize]); reshape(imG, [1 fullSize]); reshape(imB, [1 fullSize])];
    vectIm = [vectIm, tempvectIm];
end

% mu - mean
% cvr - covariance
% wi - component weight

K = 0;
for c = 1:numel(colorModelSet)
    K = K+numel(colorModelSet{c});
end

% compute data cost according to model
logp = zeros(size(vectIm, 2), K);
k = 1;
for c = 1:numel(colorModelSet)
    colorModel = colorModelSet{c};
    for ki=1:numel(colorModelSet{c})
        cvr = colorModel{ki}.cvr;
        if rank(cvr) < 3
            logp(:,k) = inf;
            continue;
        end
        wi = colorModel{ki}.wi;
        mu = colorModel{ki}.mu;

        df = vectIm';
        df(:, 1) = df(:, 1) - mu(1,1);
        df(:, 2) = df(:, 2) - mu(1,2);
        df(:, 3) = df(:, 3) - mu(1,3);

        logp(:,k) = -log(wi)+.5*log(det(cvr)) + ...
            .5*sum( df*inv(cvr).*df, 2 );
        k = k+1;
    end
end
logp = min(logp, [], 2);

logpSet = {};
ind = 1;
for i = 1:numel(labimSet)
    labim = labimSet{i};
    imsiz = size(labim);
    fullSize = prod(imsiz(1:2));
    tempLogp = logp(ind:(ind+fullSize-1));
    logpSet{i} = reshape(tempLogp, imsiz(1:2));
    ind = ind+fullSize;
end
