function [LSet, mSLset, shapeSegModels] = CoGrabCut2(mScaleImSet, initMapSet, working_add, saturation, activations, numCluster, nTransform, numOrient, templateTransform, outRow, outCol, partSizeX, partSizeY, border, bUseUncoveredPrior)
% constants:
Gamma = 30;
Beta = 1;
Eta = 0.01;
MaxItr = 20;
K = 5; % number of components in each mixture model
INF = 1000;

imSet = {};
for img = 1:numel(mScaleImSet)
    imSet{img} = double(mScaleImSet{img}{end});
end

labimSet = imSet;

vCSet = {};
hCSet = {};
scSet = {};
LSet = {};
oLSet = {};
certain_bg_set = {};
for i = 1:numel(labimSet)
    labim = labimSet{i};
    [hC vC] = SmoothnessTerm(labim);
    hCSet{i} = hC;
    vCSet{i} = vC;
    scSet{i} = Gamma.*[0 1; 1 0];
    
    initMap = initMapSet{i};
    certain_bg = initMap == -1;
    certain_bg_set{i} = certain_bg;

    oL = initMap;

    % how to label all the uncertain pixels?
%     if (sum(sum(abs(certain_fg))) == 0)
%         oL(initMap==0) = 1;
%     else
%         oL(initMap==0) = -1;
%     end
    
    LSet{i} = oL;
    oLSet{i} = oL;
end


% intialize the shape model for each texton
shapeSegModels = cell(numCluster,1);
fgColorModelGlobal = genColorModel(LSet, mScaleImSet, activations, K, 1, nTransform, numOrient, templateTransform, partSizeX, partSizeY);
bgColorModelGlobal = GenLocalColorModel(labimSet, K, LSet, -1);
for cc = 1:numCluster
    shapeSegModels{cc} = repmat(0.5, partSizeX, partSizeY);
end
uncoveredSegPrior = 0.5;

% iteration
done_suc = false;
for itr=1:MaxItr,
    % segment the imSet by the given shape&color models and shape prior map
    LSet = {};
    for i = 1:numel(labimSet)
        % GMM for foreground and background - global model
        if sum(oLSet{i}(:)==1)>K
            fgColorModelImage = GenLocalColorModel(labimSet(i), K, oLSet(i), 1);
            logpFGSet = ApplyLocalColorModel(labimSet(i), {fgColorModelImage,fgColorModelGlobal}); 
        else
            logpFGSet = ApplyLocalColorModel(labimSet(i), {fgColorModelGlobal}); 
        end
        
        
        bgColorModelImage = GenLocalColorModel(labimSet(i), K, oLSet(i), -1);
        logpBGSet = ApplyLocalColorModel(labimSet(i), {bgColorModelImage,bgColorModelGlobal}); 
    
        logpFG = logpFGSet{1};
        logpBG = logpBGSet{1};
        
        % gen shape prior map
        ind = find( activations(1,:) == i );
        shapePriorMap = genshapePriorMap(mScaleImSet{i}, activations(:,ind), shapeSegModels, uncoveredSegPrior, nTransform, numOrient, templateTransform, border);
        logpFG = logpFG - Beta*log(shapePriorMap);
        logpBG = logpBG - Beta*log(1-shapePriorMap);
        
        logpFG(isinf(logpFG)) = INF;
        logpBG(isinf(logpBG)) = INF;
        
        certain_bg = certain_bg_set{i};
        logpFG(certain_bg) = INF;

        dc = cat(3, logpBG, logpFG);
        
        sc = scSet{i};
        hC = hCSet{i};
        vC = vCSet{i};
        oL = oLSet{i};

        gch = GraphCut('open', dc , sc, vC, hC);  
        gch = GraphCut('set', gch, int32(oL==1)); % initial guess - previous result
        [gch L] = GraphCut('expand', gch);
        L = (2*L-1); % convert {0,1} to {-1,1} labeling
        gch = GraphCut('close', gch);
        
        % force labeling of certain labeling
%         certain_bg = certain_bg_set{i};
%         L(certain_bg) = -1;
        
        LSet{i} = L;
    end

    % stop if converged
    change = 0; pixelSum = 0;
    for i = 1:numel(labimSet);
        change = change + sum(oLSet{i}(:)~=LSet{i}(:));
        pixelSum = pixelSum + numel(LSet{i});
    end
    if change < .001*pixelSum
        done_suc = true;
        break;
    end
    oLSet = LSet;
    
    % relearn the shape and color model
    croppedSegMapSet = {};
    mSLset = cell(numel(mScaleImSet),1);
    for img = 1:numel(mScaleImSet)
        for s = 1:numel(mScaleImSet{img})
            mSLset{img}{s} = imresize(LSet{img},[size(mScaleImSet{img}{s},1),size(mScaleImSet{img}{s},2)],'nearest');
        end
    end
    croppedSegMaps = cropSegmentationMap(mSLset, activations, numCluster, shapeSegModels, nTransform, numOrient, templateTransform, outRow, outCol);
    
    fgColorModelGlobal = genColorModel(LSet, mScaleImSet, activations, K, 1, nTransform, numOrient, templateTransform, partSizeX, partSizeY);
    bgColorModelGlobal = GenLocalColorModel(labimSet, K, LSet, -1);
    for cc = 1:numCluster
        dim = ndims(croppedSegMaps{cc}{1});          %# Get the number of dimensions for your arrays
        M = cat(dim+1,croppedSegMaps{cc}{:});        %# Convert to a (dim+1)-dimensional matrix
        meanArray = mean(M,dim+1);
        shapeSegModels{cc} = (meanArray+1)/2;
        shapeSegModels{cc}(shapeSegModels{cc} < Eta) = Eta;
        shapeSegModels{cc}(shapeSegModels{cc} > 1-Eta) = 1-Eta;
    end
    
    if bUseUncoveredPrior
        meanUncovered = zeros(numel(LSet),1);
        for i = 1:numel(LSet)
            L = LSet{i};
            initMap = initMapSet{i};
            labelUncovered = L(initMap == 0);
            meanUncovered(i) = mean(labelUncovered);
        end
        uncoveredSegPrior = (mean(meanUncovered)+1)/2;
        uncoveredSegPrior(uncoveredSegPrior < Eta) = Eta;
        uncoveredSegPrior(uncoveredSegPrior > 1-Eta) = 1-Eta;
    end
end
if ~done_suc
    warning('GrabCut:GrabCut','Failed to converge after %d iterations', itr);
end






