disp(['===============> Getting parameters, mex-c code, and training images']);

mkdir(working_add); % working directory to store .mat files. 
mkdir(output_dir); % directory that contains output from the code. 

%% read in positive images
sizeTemplatex = templateSize(1);
sizeTemplatey = templateSize(2);
halfTemplatex = floor(sizeTemplatex/2);
halfTemplatey = floor(sizeTemplatey/2);

imageName = dir([imageFolder '/*.bmp']);
imageName = [imageName; dir([imageFolder '/*.jpg'])];
imageName = [imageName; dir([imageFolder '/*.png'])];
numImage = size(imageName, 1); % number of training images 
numImageTrain = numImage;
Ioriginal = cell(1, numImage);
numCI=zeros(numImage,numCluster);       % number of templates per image
numPOS=0;                               % number of positive images
totalACT=0;                             % total number of activated templates
totalL=0;                               % total log-likelihood
totalR=zeros(numImage,1);               % total area covered by the selected templates
totalSumR=zeros(numImage,1); 

for img = 1 : numImage
    tmpImColor = imread([imageFolder '/' imageName(img).name]); 
    if size(tmpImColor,3) == 3
        tmpIm = rgb2gray(tmpImColor);
    else
        tmpIm = tmpImColor;
    end

    sx = size(tmpIm,1); sy = size(tmpIm,2);
    if resizeTrainingImages
        if (resizeFactor < 0.)
            resizeFactor = sqrt(constantImageArea/(sx*sy))
        end
    	tmpIm = imresize( tmpIm, resizeFactor, 'bilinear' );
        tmpImColor = imresize( tmpImColor, resizeFactor, 'bilinear' );
    end
    Ioriginal{img} = single(tmpIm);
    J0 = Ioriginal{img};
    J = cell(1, numResolution);
    Jc = cell(1, numResolution);
    for r=1:numResolution
       J{r} = imresize(J0, allResolution(r), 'nearest');  % scaled images
       Jc{r} = imresize(tmpImColor, [size(J{r},1),size(J{r},2)], 'nearest');
    end
    multipleResolutionImageName = [working_add '/multipleResolutionImage' num2str(img)]; 
    save(multipleResolutionImageName, 'J', 'Jc');
end

count = 0;
% generate the set of geometric transformations for each template
for templateScaleInd = 0:0 % large scale change
    for rotation = rotationRange 
        for rowScale = 2.^[0] % small scale change and aspect ratio change
            for colScale = 2.^[0]
                for reflection = reflectionRange
                    count = count + 1;
                    templateTransform{count} = [templateScaleInd rowScale colScale rotation reflection];
                end
            end
        end
    end
end
nTransform = count;

% prepare morph-back mappings for geometric transforms
largestPartSizeX = templateSize(1); largestPartSizeY = templateSize(2);
denseX = -floor(largestPartSizeX/2) + (1:largestPartSizeX);
denseY = -floor(largestPartSizeY/2) + (1:largestPartSizeY);
count = 0;
inRow = zeros(length(denseX)*length(denseY),1,'single');
inCol = zeros(length(denseX)*length(denseY),1,'single');
inO = zeros(numel(inRow),1,'single'); 
inS = zeros(numel(inRow),1,'single');
for y = denseY
    for x = denseX
        count = count+1;
        inRow(count) = x;
        inCol(count) = y;
    end
end
outRow = cell(nTransform,1);
outCol = cell(nTransform,1);
for iT = 1:nTransform
	tScale = templateTransform{iT}(1); 
	rScale = templateTransform{iT}(2);
	cScale = templateTransform{iT}(3);
	[outRow{iT}, outCol{iT}] = ...
		mexc_TemplateAffineTransform(tScale,rScale,cScale,...
		    templateTransform{iT}(4),inRow,inCol,inO,inS,numOrient);
end

save([working_add,'/partLocConfig.mat'],'templateSize','scales','numRandomStart','maxNumClusterMember', 'resizeFactor', ...
	'S2Thres','S1softthres','allResolution','resizeTrainingImages','constantImageArea',...
    'category','numOrient','localOrNot','subsample','saturation','locationShiftLimit','orientShiftLimit',...
    'numElement','thresholdFactor','doubleOrNot', 'numCluster', 'numIter', 'subsampleS2', 'locationPerturbationFraction',...
    'partSize','rotationRange','nTransform','templateTransform','S1softthres','subsampleM1','localHalfx','localHalfy',...
    'inRow', 'inCol', 'outRow', 'outCol', 'largestPartSizeX', 'largestPartSizeY','locationPerturbationFraction_final');
clear Ioriginal

