function shapePriorMap = genshapePriorMap(Jc, activations, segshapeTemplates, uncoveredSegPrior, nTransform, numOrient, templateTransform, border)
% initialize the mask of bounding boxes and sketched image
INF = 1000;
shapePriorMap = repmat(-INF, size(Jc{end},1),size(Jc{end},2)); % the bounding box map is of the highest resolution
scoreMap = repmat(-INF, size(Jc{end},1),size(Jc{end},2));
    
for t = 1:size(activations,2) % for each activated template (or codeword, or part) 

    iCluster = ceil( ( activations(5,t) + 1 ) / nTransform );
    iTransform = activations(5,t) + 1 - (iCluster-1) * nTransform;
    iRes = activations(4,t) + 1;

    scaling = double(size(Jc{end},1)) / double(size(Jc{iRes},1));
    partSizeX = size(segshapeTemplates{iCluster},1);
    partSizeY = size(segshapeTemplates{iCluster},2);
    
    largerPartSizeX = floor(partSizeX * scaling+.5);
    largerPartSizeY = floor(partSizeY * scaling+.5);

    [inRow, inCol] = ndgrid(-floor(largerPartSizeX/2):floor(largerPartSizeX/2), -floor(largerPartSizeY/2):floor(largerPartSizeY/2));
    inRow = single(inRow);
    inCol = single(inCol);
    inO = zeros(numel(inRow),1,'single');
    inS = zeros(numel(inRow),1,'single');
    tScale = templateTransform{iTransform}(1); 
	rScale = templateTransform{iTransform}(2);
	cScale = templateTransform{iTransform}(3);
    actualPartRotation = templateTransform{iTransform}(4);
    reflection = templateTransform{iTransform}(5);
    
    if reflection < 0
        inCol = -inCol+1;
    end
    
    inRow = reshape(inRow, numel(inRow), 1);
    inCol = reshape(inCol, numel(inCol), 1);

    [outRow, outCol] = ...
        mexc_TemplateAffineTransform(tScale,rScale,cScale,...
            actualPartRotation,inRow,inCol,inO,inS,numOrient);

    

    outRow = floor(.5 + outRow + activations(2,t)*scaling);
    outCol = floor(.5 + outCol + activations(3,t)*scaling);

    invalid = (outRow <= 1) | (outRow >= size(shapePriorMap,1)) | (outCol <= 1) | (outCol >= size(shapePriorMap,2));
    posRow = floor((inRow+floor(largerPartSizeX/2))/ scaling+.5);
    posCol = floor((inCol+floor(largerPartSizeY/2)) / scaling+.5);
    invalid = invalid | (posRow < 1) | (posRow > partSizeX) | (posCol < 1) | (posCol > partSizeY);
    
    score = activations(6,t);
    outRow = outRow(invalid==0);
    outCol = outCol(invalid==0);
    posRow = posRow(invalid==0);
    posCol = posCol(invalid==0);
    ind = sub2ind(size(scoreMap),outRow,outCol);
    invalid2 = scoreMap(ind) > score;
    outRow = outRow(invalid2==0);
    outCol = outCol(invalid2==0);
    posRow = posRow(invalid2==0);
    posCol = posCol(invalid2==0);
    
    cur_segshapeTemplate = segshapeTemplates{iCluster};
    for i = 1:length(outRow)
        cur_outRow = outRow(i);
        cur_outCol = outCol(i);
        cur_posRow = posRow(i);
        cur_posCol = posCol(i);  
        
        for tx = -1:1:1
            for ty = -1:1:1
                shapePriorMap(cur_outRow+tx, cur_outCol+ty) = max(shapePriorMap(cur_outRow+tx, cur_outCol+ty), cur_segshapeTemplate(cur_posRow,cur_posCol));
                scoreMap(cur_outRow+tx, cur_outCol+ty) = score;
            end
        end
    end
end

shapePriorMap(shapePriorMap == -INF) = uncoveredSegPrior;

% shapePriorMap(1:border,:) = 0.01;
% shapePriorMap(:,1:border) = 0.01;
% shapePriorMap(end-border:end,:) = 0.01;
% shapePriorMap(:,end-border:end) = 0.01;

  